
IFrameManager = {
	frameList = { },
}


local baseInterface = { }
local interfaceMetatable = { __index = baseInterface }

function baseInterface:getName(frame)
	return frame:GetName()
end

function baseInterface:getBorder(frame)
	return 0, 0, 0, 0
end

function IFrameManager:Interface()
	local iface = { }
	setmetatable(iface, interfaceMetatable)
	return iface
end

function IFrameManager:Register(frame, iface)
	if (getmetatable(iface) == interfaceMetatable) then
		self.frameList[frame] = iface
	else
		DEFAULT_CHAT_FRAME:AddMessage("wrong metatable for frame: "..frame:GetName())
	end
end

function IFrameManager:Start()
	for frame, iface in self.frameList do
		frame.IFrameManager = IFrameFactory:Create("IFrameManager", "Capsule")
		local capsule = frame.IFrameManager
		
		capsule:SetParent(frame:GetParent())
		capsule:SetScale(frame:GetScale())
		
		local t, r, b, l = iface:getBorder(frame)
		capsule:SetWidth(frame:GetWidth() + l + r)
		capsule:SetHeight(frame:GetHeight() + t + b)
		
		capsule:ClearAllPoints()
		capsule:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (frame:GetLeft() or 100) - l, (frame:GetBottom() or 100) - b)
		
		frame:ClearAllPoints()
		frame:SetPoint("CENTER", capsule, "CENTER", (l - r) / 2, (b - t) / 2)
		
		capsule.label:SetText(iface:getName(frame))
		capsule.label:SetWidth(capsule:GetWidth() - 4)
				
		capsule:Show()
	end
end

function IFrameManager:Stop()
	for frame, iface in self.frameList do
		local capsule = frame.IFrameManager
		frame.IFrameManager = nil
		
		local x, y = frame:GetLeft(), frame:GetBottom()
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x, y)
		frame:SetUserPlaced(true)
		capsule:SetUserPlaced(false)
		
		capsule:Hide()
		IFrameFactory:Destroy("IFrameManager", "Capsule", capsule)
	end
end


SLASH_IFrameManager1 = "/ifm"

SlashCmdList["IFrameManager"] = function(msg)
	if (msg == "start") then
		IFrameManager:Start()
	elseif (msg == "stop") then
		IFrameManager:Stop()
	end
end