
coolDownConfig = {
	buttonDock = "Bottom",
	iconDock = "Right",
	minDuration = 2.2,
	maxDuration = 10000,
	backdropColor = { r = 0, g = 0, b = 0, a = 1 },
	textColor = { r = 1.0, g = 0.82, b = 0, a = 1 },
}

coolDownButton = { }

local frameDockTable = {
	["Top"] = {
		point = "BOTTOM",
		relativePoint = "TOP",
		xOffset = 0,
		yOffset = -2,
	},
	["Bottom"] = {
		point = "TOP",
		relativePoint = "BOTTOM",
		xOffset = 0,
		yOffset = 2,
	},
	["Left"] = {
		point = "RIGHT",
		relativePoint = "LEFT",
		xOffset = 1,
		yOffset = 0,
	},
	["Right"] = {
		point = "LEFT",
		relativePoint = "RIGHT",
		xOffset = -1,
		yOffset = 0,
	},
}

function coolDownButton_onUpdate()
	if (not this:IsVisible()) then
		return
	end
	
	local spellRemaining = this.spellInfo.spellDuration - ( GetTime() - this.spellInfo.spellStart)
	if (spellRemaining > 0) then
		local textLabel = coolDown:formatTime(spellRemaining)
		if (textLabel) then
			this.label:SetText(textLabel)
		end
	else
		coolDown:onEvent()
	end
end


coolDown = {
	buttonList = { },
	iconList = { },
}

local iface = IFrameManager:Interface()
function iface:getName(frame)
	return "coolDown"
end

function coolDown:onLoad()
	coolDownDock:RegisterEvent("VARIABLES_LOADED")
	coolDownDock:RegisterEvent("PLAYER_ENTERING_WORLD")
	coolDownDock:RegisterEvent("UPDATE_SHAPESHIFT_FORMS")
	coolDownDock:RegisterEvent("SPELLS_CHANGED")
	coolDownDock:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	coolDownDock:RegisterEvent("CURRENT_SPELL_CAST_CHANGED")
	
	coolDownDock:SetBackdropBorderColor(0, 0, 0, 1)
	coolDownDock:SetBackdropColor(0, 0, 0, 1)
	
	coolDownDock:Hide()
	
	IFrameManager:Register(coolDownDock, iface)
end

function coolDown:validateConfig()
	if (coolDownConfig.buttonDock == nil) then
		coolDownConfig.buttonDock = "Bottom"
	end
	local buttonDirectionInfo = frameDockTable[coolDownConfig.buttonDock]
	if (buttonDirectionInfo == nil) then
		coolDownConfig.buttonDock = "Bottom"
	end
	
	if (coolDownConfig.iconDock == nil) then
		coolDownConfig.iconDock = "Right"
	end
	local iconDockInfo = frameDockTable[coolDownConfig.iconDock]
	if (iconDockInfo == nil) then
		coolDownConfig.iconDock = "Right"
	end
	
	if (coolDownConfig.minDuration == nil) then
		coolDownConfig.minDuration = 2.2
	end
	
	if (coolDownConfig.maxDuration == nil) then
		coolDownConfig.maxDuration = 10000
	end
	
	if (coolDownConfig.backdropColor == nil) then
		coolDownConfig.backdropColor = { r = 0, g = 0, b = 0, a = 1 }
	end
	
	if (coolDownConfig.textColor == nil) then
		coolDownConfig.textColor = { r = 1.0, g = 0.82, b = 0, a = 1 }
	end
end

local function spellInfoListHasSpell(spellInfoList, spellName)
	for _, spellInfo in spellInfoList do
		if (spellInfo.spellName == spellName) then
			return true
		end
	end
	
	return false
end

function coolDown:onEvent()
	local sortedSpells = { }
	
	local minDuration = coolDownConfig.minDuration
	local maxDuration = coolDownConfig.maxDuration
	
	local currentTime = GetTime()
	for _, spellInfo in spellBook.playerSpells do
		local spellRemaining = spellInfo.spellStart + spellInfo.spellDuration - currentTime
		if (spellInfo.spellStart and spellRemaining > 0 and
			spellInfo.spellDuration > minDuration and
			spellInfo.spellDuration < maxDuration and
			not IsCurrentCast(spellInfo.spellID, "spell")) then
			local haveSpell = false
			local haveSpellList = nil
			
			for _, spellInfoList in sortedSpells do
				haveSpell = spellInfoListHasSpell(spellInfoList, spellInfo.spellName)
				if (haveSpell == true) then
					break
				else
					if (spellInfo.spellStart == spellInfoList[1].spellStart and 
						spellInfo.spellDuration == spellInfoList[1].spellDuration) then
						haveSpellList = spellInfoList
					end
				end
			end
			
			if (haveSpellList) then
				table.insert(haveSpellList, spellInfo)
			elseif (haveSpell == false) then
				local haveSpellList = { }
				table.insert(haveSpellList, spellInfo)
				table.insert(sortedSpells, haveSpellList)
			end
		end
	end
	
	for _, spellInfo in spellBook.petSpells do
		local spellRemaining = spellInfo.spellStart + spellInfo.spellDuration - currentTime
		if (spellInfo.spellStart and spellRemaining > 0 and spellInfo.spellDuration > minDuration) then
			local haveSpell = false
			local haveSpellList = nil
			
			for _, spellInfoList in sortedSpells do
				haveSpell = spellInfoListHasSpell(spellInfoList, spellInfo.spellName)
				if (haveSpell == true) then
					break
				else
					if (spellInfo.spellStart == spellInfoList[1].spellStart and 
						spellInfo.spellDuration == spellInfoList[1].spellDuration) then
						haveSpellList = spellInfoList
					end
				end
			end
			
			if (haveSpellList) then
				table.insert(haveSpellList, spellInfo)
			elseif (haveSpell == false) then
				local haveSpellList = { }
				table.insert(haveSpellList, spellInfo)
				table.insert(sortedSpells, haveSpellList)
			end
		end
	end
	
	table.sort(sortedSpells,
		function(leftEntry, rightEntry)			
			local leftStart, leftDuration = GetSpellCooldown(leftEntry[1].spellID, leftEntry[1].spellType)
			local rightStart, rightDuration = GetSpellCooldown(rightEntry[1].spellID, rightEntry[1].spellType)
			
			local leftTime = leftDuration - ( currentTime - leftStart)
			local rightTime = rightDuration - ( currentTime - rightStart)
			
			return leftTime < rightTime
		end
	)
	
	inCoolDown = true
	
	for _, frameObject in ipairs(self.buttonList) do
		IFrameFactory:Destroy("coolDown", "Button", frameObject)
	end
	
	self.buttonList = { }
	
	for _, frameObject in ipairs(self.iconList) do
		IFrameFactory:Destroy("coolDown", "Icon", frameObject)
	end
	
	self.iconList = { }
	
	local frameParent = coolDownDock
	
	coolDown:validateConfig()

	local buttonDockInfo = frameDockTable[coolDownConfig.buttonDock]
	local iconDockInfo = frameDockTable[coolDownConfig.iconDock]
	
	for buttonIndex, spellInfoList in sortedSpells do
		local buttonFrame = IFrameFactory:Create("coolDown", "Button")
		table.insert(self.buttonList, buttonFrame)
		buttonFrame.spellInfo = spellInfoList[1]
		
		buttonFrame:ClearAllPoints()
		buttonFrame:SetPoint(buttonDockInfo.point, frameParent, buttonDockInfo.relativePoint, buttonDockInfo.xOffset, buttonDockInfo.yOffset)
		
		local iconParent = buttonFrame
		
		for _, spellInfo in spellInfoList do
			local iconFrame = IFrameFactory:Create("coolDown", "Icon")
			table.insert(self.iconList, iconFrame)
			
			iconFrame:ClearAllPoints()
			iconFrame:SetPoint(iconDockInfo.point, iconParent, iconDockInfo.relativePoint, iconDockInfo.xOffset, iconDockInfo.yOffset)
			iconFrame.texture:SetTexture(spellInfo.spellTexture)
			iconFrame:SetParent(buttonFrame)
			
			iconParent = iconFrame
		end
		
		frameParent = buttonFrame
	end
	
	inCoolDown = false
end


--[[
	coolDownButton Factory
]]
local coolDownButtonFactory = { }
IFrameFactory:Register("coolDown", "Button", coolDownButtonFactory)

function coolDownButtonFactory:Create(name)
	local frame = CreateFrame("Frame", name, UIParent)
	frame:SetWidth(74)
	frame:SetHeight(32)
	
	local backdropTable = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true,
		tileSize = 12,
		edgeSize = 12,
		insets = {
			left = 2,
			right = 2,
			top = 2,
			bottom = 2
		}
	}
	
	local backdropColor = coolDownConfig.backdropColor
	local textColor = coolDownConfig.textColor
	
	frame:SetBackdrop(backdropTable)
	frame:SetBackdropBorderColor(backdropColor.r, backdropColor.g, backdropColor.b, backdropColor.a)
	frame:SetBackdropColor(backdropColor.r, backdropColor.g, backdropColor.b, backdropColor.a)	
	
	frame.label = frame:CreateFontString(name.."Label", "OVERLAY")
	frame.label:Show()
	frame.label:SetFontObject(coolDownFont)
	frame.label:ClearAllPoints()
	frame.label:SetPoint("CENTER", frame, "CENTER", -1, 0)
	frame.label:SetJustifyH("CENTER")
	frame.label:SetTextColor(textColor.r, textColor.g, textColor.b, textColor.a)
	
	frame:SetScript("OnUpdate", coolDownButton_onUpdate)
	
	return frame
end

function coolDownButtonFactory:Destroy(frame)
	frame.spellInfo = nil
	
	return frame
end


--[[
	coolDownIcon Factory
]]
local coolDownIconFactory = { }
IFrameFactory:Register("coolDown", "Icon", coolDownIconFactory)

function coolDownIconFactory:Create(name)
	local frame = CreateFrame("Frame", name, UIParent)
	frame:SetWidth(30)
	frame:SetHeight(30)
	
	frame.texture = frame:CreateTexture(name.."Texture", "ARTWORK")
	frame.texture:SetWidth(28)
	frame.texture:SetHeight(28)
	frame.texture:Show()
	frame.texture:ClearAllPoints()
	frame.texture:SetPoint("CENTER", frame, "CENTER", 0, 0)
	
	frame.border = frame:CreateTexture(name.."Border", "OVERLAY")
	frame.border:SetWidth(30)
	frame.border:SetHeight(30)
	frame.border:SetTexture("Interface\\Buttons\\UI-Debuff-Border")
	frame.border:Show()
	frame.border:ClearAllPoints()
	frame.border:SetPoint("CENTER", frame, "CENTER", 0, 0)
	frame.border:SetVertexColor(0, 0, 0, 1)
	
	return frame
end

function coolDownIconFactory:Destroy(frame)
	return frame
end

function coolDown:formatTime(timeLeft)
	local minutes = math.floor(timeLeft / 60)
	local seconds = math.floor(math.mod(timeLeft, 60))
	
	if (this.savedMinutes == minutes and this.savedSeconds == seconds) then
		return nil
	end
	
	this.savedMinutes = minutes
	this.savedSeconds = seconds
	
	return string.format("%02d:%02s", minutes, seconds)
end

